import java.awt.*;

/*****************************************************************************
* File: StatusBar.java  Name:StatusBar
* Goal: To display text areas at the bottom of a panel.  It will inclose 
*       these text areas inside of box which it will draw using the
*       rectangle primitives.
*
* Written: Bradley Huffaker             (12/17/97)
*
* For:Cooperative Association for Internet Data Analysis
******************************************************************************
******************************************************************************
*  By accessing this software, STATUSBAR, you are duly informed of
*  and agree to be bound by the conditions described below in
*  this notice:
*
*  This software product, STATUSBAR, is developed by Bradley Huffaker, and
*  copyrighted(C) 1998 by the University of California, San Diego (UCSD),
*  with all rights reserved.  UCSD administers the NSF grant to CAIDA,
*  number NCR-9711092, under which this code was developed. 
*
*  STATUSBAR is a free software. You can redistribute it and/or modify it
*  under the terms of the CAIDA General Public License v.1 as
*  published by UCSD and is incorporated by reference herein.  STATUSBAR is
*  distributed WITHOUT ANY WARRANTY, IMPLIED OR EXPRESS, OF MERCHANTABILITY
*  OR FITNESS FOR A PARTICULAR PURPOSE or that the use of it will not
*  infringe on any third party's intellectual property rights.
*
*  You should have received a copy of the CAIDA General Public
*  License v.1 along with the STATUSBAR program.  If for any reason you
*  have not received a copy, please write to
*
*                       University of California, San Diego
*                       SDSC/CAIDA
*                       9500 Gilman Dr., MS-0505
*                       La Jolla, CA 92093 - 0505  USA
*
*  Or contact INFO@CAIDA.ORG
*****************************************************************************/ 

public class StatusBar extends Panel
{

 // The current string to be displayed in each slot
 protected String[] strings; 

 // For now this will only work for the last stdin
 // This will allow the box to grow so that by wrapping the last string 
 // around we can display the whole thing.  Hope it doesn't look too horrible
 // 
 // wrap_strings will be set to null every time a value in strings is changed
 // That way they will be recalculated when before they are draw.  I have to
 // do it in the draw.  Because that is the only part where I am certain 
 // to have the canvas set already, allowing me to get its dimensions and fonts.
 protected String[] wrap_strings;

 // The length of the longest string so far displayed in each slot
 protected int[] lengths;
 protected int last_width = 0;

 // This are the colors which are used for the box effect.
 static final protected Color LIGHT = new Color(240,240,240);
 static final protected Color TOP = Color.lightGray;
 static final protected Color BOTTOM = Color.white;
 static final protected Color DARK = Color.darkGray;

 // This is the color of the text
 protected Color string_color = Color.black;

 // The size of the slop on the Box
 static final protected int SLOP_X = 2;
 static final protected int FLAT_X = 2;
        
 static final protected int SLOP_Y = 2;
 static final protected int FLAT_Y = 2;

 // This is the total height that the scroll bar will be if know
 protected int current_height = 0;

 // This the the parent_component on which the status bar will be drawn
 protected Component parent_component;

 // parents 
 protected FontMetrics fm;

 public StatusBar(Component parent)
 {
        current_height = 0;
        parent_component = parent;
        strings = new String[1];
        lengths = new int[1];

        fm = parent_component.getFontMetrics(parent_component.getFont());
 }

 /***************************************************************
 * Purpose: To set up the arrays.  It uses the value of 
 *      of integer to initialize the lengths values and to 
 *      get the size to start with.
 ****************************************************************/ 
 public StatusBar(int[] input_lengths)
 {
        strings = new String[input_lengths.length];
        lengths = new int[input_lengths.length];
        for(int index=0;index < input_lengths.length;index++)
                lengths[index] = input_lengths[index];
        wrap_strings = null;
 }

 /**************************************************************
 * Purpose: To set the strings value at index to string.
 * Calls: checkSize() to grow the array if needed.
 *
 * Special: I would like to be able to check to reset length
 *      here.  But because I can't access the Font Metrics
 *      I will have to wait to check it every time in paint!
 ***************************************************************/
 public void setString(String string, int index)
 {

        checkSize(index);
        strings[index] = string;
	//if (lengths[index] < fm.stringWidth(string) + fm.charWidth(' '))
          lengths[index] = fm.stringWidth(string) + fm.charWidth(' ');

        wrap_strings = null;
 }

 /*************************************************************
 * Purpose: Returns the string at index.
 **************************************************************/
 public String getString(int index)
 {
        if (index >= strings.length)
                return null;
        return strings[index];
 }
        

 /*************************************************************
 * Purpose: To set the length of index
 * Calls: checkSize() to grow the array if needed.
 **************************************************************/
 public void setLength(int length, int index)
 {
        checkSize(index);
        lengths[index] = length;
        wrap_strings = null;
 }

 /*************************************************************
 * Purpose: To return the length at index
 **************************************************************/
 public int getLength(int index)
 {
        if (index >= lengths.length)
                return 0;
        return lengths[index];
 }

 /*************************************************************
 * Purpose: To set the size of the arrays
 * Calls: checkSize() to grow the array if needed
 **************************************************************/
 public void setNumber(int number)
 {
        if (number < lengths.length)
        {
                String[] temp_s = new String[number];
                int[] temp_l = new int[number];
                for (int index=0; index < number; index++)
                {
                        temp_s[index] = strings[index];
                        temp_l[index] = lengths[index];
                }
                strings = temp_s;
                lengths = temp_l;
        }
        else
                checkSize(number-1);
 }

 /*************************************************************
 * Purpose: To get the size of the array
 **************************************************************/
 public int getNumber()
 {
        return strings.length;
 }

 /**************************************************************
 * Purpose: To return the higth that the scroll bar will be, if know
 ***************************************************************/
 public int getHeight()
 {
        return current_height;
 }

 /************************************************************
 * Purpose: to check to passed location is current in the array.
 *      and to expanded to array to include it if it is not.
 *************************************************************/ 
 public void checkSize(int location)
 {
        if (location >= lengths.length)
        {
                String[] temp_s = new String[location+1];
                int[] temp_l = new int[location+1];
                for (int index=0; index < lengths.length; index++)
                {
                        temp_s[index] = strings[index];
                        temp_l[index] = lengths[index];
                }
                strings = temp_s;
                lengths = temp_l;
        }
 }

 /*************************************************************
 * Purpose: To draw the strings in boxs of size length.
 **************************************************************/
 synchronized public void draw(Graphics g)
 {
        Dimension dim = parent_component.getSize();

        int padX = fm.charWidth(' '); 
        int padY = fm.getHeight();
        
        int y_shift = FLAT_Y + SLOP_Y;
        int x_shift = FLAT_X + SLOP_Y;
        int box_width = dim.width;
        int box_x = 0;

        // This will check to see if the strings have been changed
        // If they have then they will be reset.
        // null means that the wrap strings need to be reset.
        if (wrap_strings == null || dim.width !=last_width)
        {
                wrap_strings = CreateWrapStrings(dim, fm);
                if (wrap_strings == null) 
                        System.err.println("StatusBar:wrap_strings is null");
                last_width = dim.width;
        }

        int box_height = (5*padY*wrap_strings.length)/4 + 2*y_shift+padY/4;
        int box_hole_height = box_height - 2*y_shift;
        int box_y = dim.height - box_height;

        // First draw the raised level
        drawHill(g,box_x,box_y,box_width,box_height);
        box_y += y_shift;
        for (int index=0; index < strings.length; index++)
        {
                String string = strings[index];
                if (string != null && lengths[index] < fm.stringWidth(string)+padX)
                        lengths[index] = fm.stringWidth(string)+padX;
                box_x += x_shift;
                box_width = lengths[index];
                
                // Makes the last one fill all the way across
                if (index == strings.length-1)// && dim.width > box_x + box_width)
                        box_width = dim.width - x_shift - box_x;
                // Draw the hole for the string to side in
                drawHole(g, box_x, box_y, box_width, box_hole_height);

                // Draw the string inside of each hole 
                if (string != null && index < strings.length-1)
                {
                        g.setColor(string_color);
                        g.drawString(string,box_x+padX/2,box_y + padY);
                }
                // Draw the last string in stages
                else
                {
                        g.setColor(string_color);
                        int string_x = box_x + padX/2;
                        int string_y = box_y + padY;
                        if (wrap_strings != null )
                        {
                          for(int i=0; i < wrap_strings.length;i++)
                          {
                                String str = wrap_strings[i];
                                if (str != null)
                                        g.drawString(str,string_x,string_y);
                                string_y += (5*padY)/4;
                          }
                        }
                }
                                
                box_x += box_width;
        }

        if (box_height < current_height && current_height > 0)
        {
                parent_component.repaint(0, dim.height - current_height,
                        dim.width, current_height - box_height);
        }

        // Sets the total height
        current_height = box_height;
 }

 /*************************************************************
 * Purpose: To draw a raise area
 *      Starting at x,y and of size width, height
 **************************************************************/
 public void drawHill(Graphics g, int x, int y, int width, int height)
 {
        drawBox(g,LIGHT,TOP,DARK, x, y, width, height);
 }

 /*************************************************************
 * Purpose: To draw a lowered area
 *      Starting at x,y and of size width, height
 **************************************************************/
 public void drawHole(Graphics g, int x, int y, int width, int height)
 {
        drawBox(g,DARK,BOTTOM,LIGHT, x, y, width, height);
 }

 /*************************************************************
 * Purpsoe to draw a two colored box
 **************************************************************/
 protected void drawBox(Graphics g, Color left, Color middle, Color right
        , int x, int y , int width, int height)
 {
        int[] x_top = {x, x+width, x+width-SLOP_X, x+SLOP_X, x+SLOP_X,        
                x};
        int[] y_top = {y, y,       y+SLOP_Y,       y+SLOP_Y, y+height-SLOP_Y, 
                y+height};
        int[] x_bot = {x,        x+SLOP_X,        x+width-SLOP_X,  
                x+width-SLOP_X, x+width, x+width};
        int[] y_bot = {y+height, y+height-SLOP_Y, y+height-SLOP_Y, 
                y+SLOP_Y,       y,       y+height};
        g.setColor(left);
        g.fillPolygon( x_top, y_top, 6);
        g.setColor(right);
        g.fillPolygon( x_bot, y_bot, 6);
        x += SLOP_X;
        width -= 2*SLOP_X;
        y += SLOP_Y;
        height -= 2*SLOP_Y;
        g.setColor(middle);
        g.fillRect(x,y,width,height);
 }

 /***********************************************************
 * Purpose to break the last sting in strings into smaller 
 * strings which can each fit in the amound of space left for the wrapper
 ************************************************************/
 protected String[] CreateWrapStrings(Dimension dim, FontMetrics fm)
 {
        final int DIVIDER_SIZE = (2*SLOP_X + FLAT_X);
        int box_width = dim.width - 2*DIVIDER_SIZE;
        String[] answer;
        final int FINAL_STRING = strings.length-1;
        for(int index=0; index < FINAL_STRING; index++)
        {
                box_width -= lengths[index] + DIVIDER_SIZE;
        }
        int length = lengths[FINAL_STRING];
        String string = strings[FINAL_STRING];
 	if (string == null)
 		string = "";

        final char[] chars = string.toCharArray();

        if (box_width <= 0 || ((float)length)/box_width <= 1 
                || chars.length < 1)
        {
                answer = new String[1];
                if (string != null)
                        answer[0] = string;
                else
                        answer [0] = "";
        }
        else
        {
                answer = new String[(int)(length/box_width)];
                int char_start = 0;
                int char_end =1;
                int current_string = 0;
                int current_length= 0;
                while (char_end < chars.length)
                {
                        int char_length = fm.charWidth(chars[char_end]);
                        if (current_length +char_length >= box_width)
                        {
                                if (current_string == answer.length)
                                {
                                 String[] temp = answer;
                                 answer = new String[temp.length*2];
                                 for (int i=0;i<temp.length;i++)
                                  answer[i] = temp[i];
                                }
                                answer[current_string] = new String(chars,
                                        char_start, char_end - char_start);
                                char_start = char_end;
                                current_string++;
                                current_length=0;
                        }
                        current_length += char_length;
                        char_end++;
                }
                if (current_string == answer.length)
                {
                 String[] temp = answer;
                 answer = new String[temp.length*2];
                 for (int i=0;i<temp.length;i++)
                  answer[i] = temp[i];
                }
                answer[current_string] = new String(chars,char_start
                        , char_end-char_start);
                current_string++;
                if (answer.length != current_string)
                {
                        String[] temp = answer;
                        answer = new String[current_string];
                        for (int i=0; i < current_string; i++)
                                answer[i] = temp[i];
                }
        }
        return answer;
 }      
}
